package iot.system.iotserver.config;

import iot.system.iotserver.message.ExchangeMessage;
import iot.system.iotserver.message.server.ServerMessage;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.apache.kafka.common.serialization.StringSerializer;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.kafka.annotation.EnableKafka;
import org.springframework.kafka.config.ConcurrentKafkaListenerContainerFactory;
import org.springframework.kafka.core.*;
import org.springframework.kafka.support.serializer.JsonDeserializer;
import org.springframework.kafka.support.serializer.JsonSerializer;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

@EnableKafka
@Configuration
public class KafkaConfiguration {

    @Bean
    public ConsumerFactory<String, ExchangeMessage> messageConsumerFactory(){
        Map<String, Object> config = new HashMap<>();
        config.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, CmdLineArgs.getKafkaServers());
        config.put(ConsumerConfig.GROUP_ID_CONFIG, CmdLineArgs.getConsumerGroup());
        config.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class);
        config.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, JsonDeserializer.class);
        return new DefaultKafkaConsumerFactory<>(config,new StringDeserializer(),new JsonDeserializer<>(ExchangeMessage.class));
    }

    @Bean
    public ConcurrentKafkaListenerContainerFactory<String, ExchangeMessage> kafkaListenerContainerFactory(){
        ConcurrentKafkaListenerContainerFactory<String, ExchangeMessage> factory = new ConcurrentKafkaListenerContainerFactory<>();
        factory.setConsumerFactory(messageConsumerFactory());
        return factory;
    }

    @Bean
    public ConsumerFactory<String, ServerMessage> interServerMessageConsumerFactory(){
        Map<String, Object> config = new HashMap<>();
        config.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, CmdLineArgs.getKafkaServers());
        config.put(ConsumerConfig.GROUP_ID_CONFIG, "server-"+CmdLineArgs.getServerID());
        config.put(ConsumerConfig.CLIENT_ID_CONFIG, "server-"+CmdLineArgs.getServerID());
        config.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class);
        config.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, JsonDeserializer.class);
        return new DefaultKafkaConsumerFactory<>(config,new StringDeserializer(),new JsonDeserializer<>(ServerMessage.class));
    }

    @Bean
    public ConcurrentKafkaListenerContainerFactory<String, ServerMessage> kafkaServerListenerContainerFactory(){
        ConcurrentKafkaListenerContainerFactory<String, ServerMessage> factory = new ConcurrentKafkaListenerContainerFactory<>();
        factory.setConsumerFactory(interServerMessageConsumerFactory());
        return factory;
    }

    @Bean
    public ProducerFactory<String, ExchangeMessage> producerFactory() {
        Map<String, Object> configProps = new HashMap<>();
        configProps.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, CmdLineArgs.getKafkaServers());
        configProps.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class);
        configProps.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, JsonSerializer.class);
        configProps.put(ProducerConfig.CLIENT_ID_CONFIG, "server-"+CmdLineArgs.getServerID());
        return new DefaultKafkaProducerFactory<>(configProps);
    }

    @Bean
    public KafkaTemplate<String, ExchangeMessage> kafkaTemplate() {
        return new KafkaTemplate<>(producerFactory());
    }
}
