package iot.system.iotserver.config;

import iot.system.iotserver.enums.ClusterConnectionState;

import javax.persistence.criteria.CriteriaBuilder;
import java.time.Instant;
import java.util.Hashtable;
import java.util.Map;

public class RuntimeConfiguration {
    private static RuntimeConfiguration instance;

    static public RuntimeConfiguration getInstance(){
        if(RuntimeConfiguration.instance == null){
            RuntimeConfiguration.instance = new RuntimeConfiguration();
        }
        return RuntimeConfiguration.instance;
    }

    private int leader;
    private ClusterConnectionState clusterConnectionState;
    private Map<Integer, Integer> topology;
    private Map<Integer, Instant> clusterActivity;
    private boolean electionRunning;
    private boolean electionParticipant;
    private final Instant startedAt;

    private RuntimeConfiguration(){
        this.leader = -1;
        this.topology = new Hashtable<>();
        this.clusterActivity = new Hashtable<>();
        this.clusterConnectionState = ClusterConnectionState.NOT_CONNECTED;
        this.electionRunning = false;
        this.electionRunning = false;
        this.startedAt = Instant.now();
    }

    public synchronized boolean isLeader() {
        return leader == CmdLineArgs.getServerID();
    }

    public synchronized void setLeader(int leader) {
        this.leader = leader;
    }

    public synchronized int getLeader() {
        return leader;
    }

    public synchronized ClusterConnectionState getClusterConnectionState() {
        return clusterConnectionState;
    }

    public synchronized void setClusterConnectionState(ClusterConnectionState clusterConnectionState) {
        this.clusterConnectionState = clusterConnectionState;
    }

    public synchronized Map<Integer, Integer> getTopology() {
        return topology;
    }

    public synchronized void setTopology(Map<Integer, Integer> topology) {
        this.topology = topology;
    }

    public synchronized Map<Integer, Instant> getClusterActivity() {
        return clusterActivity;
    }

    public synchronized void setClusterActivity(Map<Integer, Instant> clusterActivity) {
        this.clusterActivity = clusterActivity;
    }

    public synchronized boolean isElectionRunning() {
        return electionRunning;
    }

    public synchronized void setElectionRunning(boolean electionRunning) {
        this.electionRunning = electionRunning;
    }

    public synchronized boolean isElectionParticipant() {
        return electionParticipant;
    }

    public synchronized void setElectionParticipant(boolean electionParticipant) {
        this.electionParticipant = electionParticipant;
    }

    public synchronized Instant getStartedAt() {
        return startedAt;
    }

    public synchronized String topologyToString(){
        Integer leaderId = this.leader;
        Integer currentNode = leaderId;
        String result = "";
        while(true){
            if(this.topology.get(currentNode).equals(leaderId)){
                result = result + currentNode.toString() + " -> "+this.topology.get(currentNode);
                break;
            }
            else{
                result = result + currentNode.toString() + " -> ";
            }
            currentNode = this.topology.get(currentNode);
        }
        return result;
    }
}
