package iot.system.iotserver.modules;

import iot.system.iotserver.message.node.InterfaceStateMessage;
import iot.system.iotserver.model.Interface;
import iot.system.iotserver.model.Module;
import iot.system.iotserver.model.ModuleVar;
import iot.system.iotserver.support.CustomLogger;
import org.springframework.boot.logging.LogLevel;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


abstract public class AbstractModule {
    private Map<String, ModuleVar> vars;

    AbstractModule(){
        this.vars = new HashMap<>();
    }

    AbstractModule(Module module){
        this();
        for(ModuleVar var: module.getVariables()){
            this.vars.put(var.getVariable().getName(), var);
        }
    }

    public void processMessage(InterfaceStateMessage message){
        Map<String, List<Integer>> interfaceData = message.getInterfacesStates();
        for(String varName: vars.keySet()){
            ModuleVar var = this.vars.get(varName);
            if(var.getAttachedInterface()==null) continue;
            if(interfaceData.containsKey(var.getAttachedInterface().getUid())){
                this.setVarFromData(var.getVariable().getName(),interfaceData.get(var.getAttachedInterface().getUid()));
            }
        }
        this.execute();
    }

    abstract protected void setVarFromData(String varName, List<Integer> varData);

    abstract public void execute();

    protected boolean setVariableValue(String varName, String value){
        return false;
    }

    protected boolean setVariableValue(String varName, Double value){
        return false;
    }

    protected boolean setVariableValue(String varName, Integer value){
        ModuleVar var = this.vars.get(varName);
        if(var == null) return false;

        var.setValue(value.toString());
        return true;
    }

    protected String getVariableValueAsString(String varName){
        return "";
    }

    protected Double getVariableValueAsDouble(String varName){
        return 0.0;
    }

    protected Integer getVariableValueAsInteger(String varName){
        ModuleVar var = this.vars.get(varName);
        if(var == null){
            throw new RuntimeException("Unknown variable ["+varName+"]");
        }

        return Integer.parseInt(var.getValue());
    }

    protected Interface getAttachedInterfaceForVariable(String varName){
        ModuleVar var = this.vars.get(varName);
        if(var == null){
            throw new RuntimeException("Unknown variable ["+varName+"]");
        }

        if(var.getAttachedInterface() != null){
            return var.getAttachedInterface();
        }
        else{
            return null;
        }
    }

    static public AbstractModule createInstanceFromModule(Module module){
        try{
            String className = "iot.system.iotserver.modules."+module.getType().getModuleName();
            Class<?> c = Class.forName(className);
            Constructor<?> cons = c.getConstructor(Module.class);
            return  (AbstractModule)cons.newInstance(module);
        }
        catch(Exception  e){
            CustomLogger.log(e.toString(), LogLevel.ERROR);
            e.printStackTrace();
            return null;
        }
    }
}
