package iot.system.iotserver.modules;

import iot.system.iotserver.message.node.InterfaceStateMessage;
import iot.system.iotserver.model.Interface;
import iot.system.iotserver.model.Module;
import iot.system.iotserver.runners.MessageProcessor;

import java.time.Instant;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Radiator extends AbstractModule {

    public Radiator(Module module){
        super(module);
    }

    @Override
    protected void setVarFromData(String varName, List<Integer> varData) {
        if(("fanSpeed").equals(varName)){
            this.setVariableValue(varName,varData.get(2));
        }
        if(("tempIn").equals(varName)){
            this.setVariableValue(varName,varData.get(3));
        }
        if(("tempOut").equals(varName)){
            this.setVariableValue(varName,varData.get(5));
        }
        if(("manualOverride").equals(varName)){
            this.setVariableValue(varName,varData.get(0));
        }
        if(("manualRun").equals(varName)){
            this.setVariableValue(varName,varData.get(0));
        }
    }


    @Override
    public void execute() {
        Integer minDelta = this.getVariableValueAsInteger("minDelta");
        Integer maxDelta = this.getVariableValueAsInteger("maxDelta");
        Integer minStartTemp = this.getVariableValueAsInteger("minStartTemp");
        Integer minStopTemp = this.getVariableValueAsInteger("minStopTemp");

        Integer highFanSpeed = this.getVariableValueAsInteger("highFanSpeed");
        Integer lowFanSpeed = this.getVariableValueAsInteger("lowFanSpeed");

        Integer tempIn = this.getVariableValueAsInteger("tempIn");
        Integer tempOut = this.getVariableValueAsInteger("tempOut");
        Integer fanSpeed = this.getVariableValueAsInteger("fanSpeed");
        Integer delta = tempIn - tempOut;

        Integer led1 = this.getVariableValueAsInteger("led1");
        Integer led2 = this.getVariableValueAsInteger("led2");

        Integer newFanSpeed = -1;

        if((minStopTemp>=tempIn)&&(fanSpeed>0)){
            newFanSpeed = 0;
        }
        else if(minStartTemp<=tempIn){
            if((delta<minDelta)&&(!fanSpeed.equals(highFanSpeed))){
                newFanSpeed = highFanSpeed;
            }
            else if((maxDelta<delta)&&(fanSpeed!=0)){
                newFanSpeed = 0;
            }
            else if((minDelta<delta)&&(delta<maxDelta)&&(!fanSpeed.equals(lowFanSpeed))){
                newFanSpeed = lowFanSpeed;
            }
        }

        if(newFanSpeed>-1){
            List<Integer> dataForInterface = new ArrayList<>();
            dataForInterface.add(0,80);
            dataForInterface.add(1,0);
            dataForInterface.add(2,newFanSpeed);

            Interface intf = this.getAttachedInterfaceForVariable("fanSpeed");
            Map<String, List<Integer>> interfacesStates = new HashMap<>();
            interfacesStates.put(intf.getUid(),dataForInterface);

            InterfaceStateMessage message = new InterfaceStateMessage();
            message.setInterfacesStates(interfacesStates);
            message.setNodeLogin(intf.getBoard().getLogin());
            message.setTimestamp(Instant.now());

            MessageProcessor.getInstance().sendMessage("S2N",message);
        }



        InterfaceStateMessage ledMessage = new InterfaceStateMessage();
        ledMessage.setTimestamp(Instant.now());
        ledMessage.setNodeLogin(this.getAttachedInterfaceForVariable("led1").getBoard().getLogin());
        Map<String, List<Integer>> ledStates = new HashMap<>();
        ledMessage.setInterfacesStates(ledStates);
        if(!led1.equals(this.getLedStatus(tempIn, led1))){
            led1 = this.getLedStatus(tempIn,led1);
            this.setLedStatus("led1",led1,ledStates);
        }
        if(!led2.equals(this.getLedStatus(tempOut, led2))){
            led2 = this.getLedStatus(tempOut,led2);
            this.setLedStatus("led2",led2,ledStates);
        }

        if(ledStates.size()>0){
            MessageProcessor.getInstance().sendMessage("S2N",ledMessage);
        }
    }

    private Integer getLedStatus(Integer tempSensorVal, Integer ledStatus){
        if((tempSensorVal<=10)&&(ledStatus==0)){ //error state detected, warning led is not on
            return 1; //turn led on
        }
        else if((tempSensorVal>10)&&(ledStatus==1)){ //error state not detected, warning led is on
            return 0; //turn led off
        }
        else{
            return ledStatus; //do nothing
        }
    }

    private void setLedStatus(String ledName, Integer ledVal, Map<String, List<Integer>> ledStates){
        this.setVariableValue(ledName, ledVal);

        List<Integer> ledData = new ArrayList<>();
        ledData.add(0,ledVal);

        ledStates.put(this.getAttachedInterfaceForVariable(ledName).getUid(),ledData);
    }
}
