package iot.system.iotserver.services;

import iot.system.iotserver.config.CmdLineArgs;
import iot.system.iotserver.config.RuntimeConfiguration;
import iot.system.iotserver.enums.ClusterConnectionState;
import iot.system.iotserver.message.node.BoardConfigMessage;
import iot.system.iotserver.message.ExchangeMessage;
import iot.system.iotserver.message.node.HeartbeatMessage;
import iot.system.iotserver.message.node.InterfaceStateMessage;
import iot.system.iotserver.message.server.*;
import iot.system.iotserver.runners.MessageProcessor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.stereotype.Service;

@Service
public class KafkaConsumer {
    @Autowired
    private MessageProcessor messageProcessor;

    @KafkaListener(topics = "N2S", containerFactory = "kafkaListenerContainerFactory")
    public void consumeMessage(ExchangeMessage message){
        if(message instanceof InterfaceStateMessage){
            messageProcessor.processMessage((InterfaceStateMessage) message);
        }
        if(message instanceof BoardConfigMessage){
            messageProcessor.processMessage((BoardConfigMessage) message);
        }
        if(message instanceof HeartbeatMessage){
            messageProcessor.processMessage((HeartbeatMessage) message);
        }
    }

    @KafkaListener(topics = "S2S", containerFactory = "kafkaServerListenerContainerFactory")
    public void consumeServerMessage(ServerMessage message){
        if(message.getTimestamp().toEpochMilli()<RuntimeConfiguration.getInstance().getStartedAt().toEpochMilli()) return;

        boolean isConnected = (RuntimeConfiguration.getInstance().getClusterConnectionState() == ClusterConnectionState.CONNECTED);

        if((message instanceof LeaderElectionMessage)&&isConnected){
            RuntimeConfiguration.getInstance().setElectionRunning(true);
        }

        if(((message.getTargetID() != CmdLineArgs.getServerID())&&(message.getTargetID() > 0))||(message.getSenderID()==CmdLineArgs.getServerID())) return;

        if((message instanceof ConnectionMessage)&&isConnected){
            messageProcessor.processMessage((ConnectionMessage) message);
        }
        if(message instanceof ConnectionDeniedMessage){
            messageProcessor.processMessage((ConnectionDeniedMessage) message);
        }
        if(message instanceof ConnectionAcceptedMessage){
            messageProcessor.processMessage((ConnectionAcceptedMessage) message);
        }
        if((message instanceof TopologyUpdateMessage)&&isConnected){
            messageProcessor.processMessage((TopologyUpdateMessage) message);
        }
        if((message instanceof ServerHeartbeatMessage)&&isConnected){
            messageProcessor.processMessage((ServerHeartbeatMessage) message);
        }
        if((message instanceof LeaderElectionMessage)&&isConnected){
            messageProcessor.processMessage((LeaderElectionMessage) message);
        }
        if((message instanceof LeaderElectedMessage)&&isConnected){
            messageProcessor.processMessage((LeaderElectedMessage) message);
        }
    }
}
