/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package board;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

import board.io.Interface;
import board.io.InterfaceDirection;
import board.io.InterfaceType;
import board.utils.BoardUtils;
import exchange.Exchange;
import exchange.dto.InterfaceStateMessage;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.TimeUnit;

/**
 *
 * @author janzubr
 */
public class Board {
    private static final int PIN_SAMPLE_RATE_MS = 200;
    private String name = "N/A";
    private final ConcurrentHashMap<String, Interface> activeInterfaces;
    private final BoardUtils boardUtils;    
    private final ScheduledExecutorService readInputPinsService;
    
    private Board() {
        this.boardUtils = BoardUtils.getBoardUtils();
        this.activeInterfaces = new ConcurrentHashMap<>();
        
        Runtime.getRuntime().addShutdownHook(new Thread(new ShutdownThread()));
        
        this.readInputPinsService=Executors.newSingleThreadScheduledExecutor(new PinThreadFactory());
        this.readInputPinsService.scheduleAtFixedRate(new PinHandler(), 0, PIN_SAMPLE_RATE_MS, TimeUnit.MILLISECONDS);
    }
    
    public static Board getInstance() {
        return BoardHolder.INSTANCE;
    }
    
    /*METHODS*/

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public boolean openInterface(Interface intf){
        if(this.activeInterfaces.containsKey(intf.getDescriptor().getUid())) return true;

        if(this.getBoardUtils().openInterface(intf)){
            this.activeInterfaces.put(intf.getDescriptor().getUid(),intf);
            return true;
        }
        else{
            return false;
        }
    }

    public boolean writeInterface(Interface intf){
        return this.getBoardUtils().writeInterface(intf);
    }

    public boolean writeInterface(String interfaceUID, int[] data){
        Interface intf = this.activeInterfaces.get(interfaceUID);
        intf.setDataOut(data);
        return this.writeInterface(intf);
    }

    public boolean readInterface(Interface intf){
        return this.getBoardUtils().readInterface(intf);
    }

    public boolean closeInterface(Interface intf){
        if(!this.activeInterfaces.containsKey(intf.getDescriptor().getUid())) return true;

        if(this.getBoardUtils().closeInterface(intf)){
            this.activeInterfaces.remove(intf.getDescriptor().getUid(),intf);
            return true;
        }
        else{
            return false;
        }
    }

    public boolean closeAllInterfaces(){
        Iterator<Interface> it = this.activeInterfaces.values().iterator();
        while(it.hasNext()){
            Interface intf = it.next();
            this.closeInterface(intf);
        }
        return true;
    }

    /*METHODS END*/

    private BoardUtils getBoardUtils(){
        return this.boardUtils;
    }
    
    private static class BoardHolder {
        private static final Board INSTANCE = new Board();
    }
    
    private final class PinHandler implements Runnable{
        
        @Override
        public void run() {
            InterfaceStateMessage msg = new InterfaceStateMessage();
            Map<String, List<Integer>> interfaceStates = new HashMap<>();
            msg.setInterfacesStates(interfaceStates);

            Iterator<Interface> interfaces = activeInterfaces.values().iterator();
            while(interfaces.hasNext()){
                Interface intf = interfaces.next();
                if(!((intf.getDescriptor().getType()==InterfaceType.PIN)&&(intf.getDirection()==InterfaceDirection.IN))){
                    continue;
                }

                int oldVal = intf.getData()[0];
                Board.getInstance().readInterface(intf);
                int newVal = intf.getData()[0];

                if(oldVal!=newVal){
                    List<Integer> data = new ArrayList<>();
                    data.add(0,newVal);

                    interfaceStates.put(intf.getDescriptor().getUid(),data);
                }
            }

            if(interfaceStates.size()>0){
                Exchange.getInstance().pushMessage(msg);
                System.out.println("Sending data");
            }
        }
    }
    
    private class PinThreadFactory implements ThreadFactory{

        @Override
        public Thread newThread(Runnable r) {
            Thread T = new Thread(r);
            T.setDaemon(true);
            return T;
        }
        
    }
    
    private final class ShutdownThread implements Runnable{

        @Override
        @SuppressWarnings("empty-statement")
        public void run() {
            Board.getInstance().closeAllInterfaces();
            System.out.println("All interfaces closed");
        }
        
    }
}
