package board.io;

import com.fasterxml.jackson.annotation.JsonIgnore;
import exceptions.MalformedUIDException;
import sun.reflect.generics.reflectiveObjects.NotImplementedException;

import java.util.Map;

public class Interface {
    private InterfaceDescriptor descriptor;
    private int[] dataIn;
    private int[] dataOut;
    private InterfaceDirection direction;
    private Map<InterfaceSettingsParams,String> settingsParameters;

    public Interface(String UID) throws MalformedUIDException {
        this.descriptor=new InterfaceDescriptor(UID);
        this.direction = InterfaceDirection.BIDIR;
        this.dataIn = new int[0];
        this.dataOut = new int[0];
    }

    public InterfaceDescriptor getDescriptor() {
        return descriptor;
    }

    synchronized public int[] getDataIn() {
        return dataIn;
    }

    @JsonIgnore
    synchronized public byte[] getDataInBytes(){
        return Interface.convertData(this.getDataIn());
    }

    @JsonIgnore
    synchronized public void setDataIn(byte[] dataIn) {
        this.setDataIn(Interface.convertData(dataIn));
    }

    synchronized public void setDataIn(int[] dataIn){
        this.dataIn = dataIn;
    }

    synchronized public int[] getDataOut() {
        return dataOut;
    }

    @JsonIgnore
    synchronized public byte[] getDataOutBytes(){
        return Interface.convertData(this.getDataOut());
    }

    @JsonIgnore
    synchronized public void setDataOut(byte[] dataOut) {
        this.setDataOut(Interface.convertData(dataOut));
    }

    synchronized public void setDataOut(int[] dataOut){
        this.dataOut = dataOut;
    }

    @JsonIgnore
    synchronized public int[] getData(){
        if(this.direction == InterfaceDirection.IN){
            return this.getDataIn();
        }
        else if(this.direction == InterfaceDirection.OUT){
            return this.getDataOut();
        }
        else{
            throw new NotImplementedException();
        }
    }

    @JsonIgnore
    synchronized public byte[] getDataBytes(){
        return Interface.convertData(this.getData());
    }

    synchronized public void setData(int[] data){
        if(this.direction == InterfaceDirection.IN){
            this.setDataIn(data);
        }
        else if(this.direction == InterfaceDirection.OUT){
            this.setDataOut(data);
        }
        else{
            throw new NotImplementedException();
        }
    }

    synchronized public void setData(byte[] data){
        this.setData(Interface.convertData(data));
    }

    synchronized public InterfaceDirection getDirection() {
        return direction;
    }

    synchronized public void setDirection(InterfaceDirection direction) {
        this.direction = direction;
    }

    synchronized public Map<InterfaceSettingsParams, String> getSettingsParameters() {
        return settingsParameters;
    }

    synchronized public void setSettingsParameters(Map<InterfaceSettingsParams, String> settingsParameters) {
        this.settingsParameters = settingsParameters;
    }

    private static int[] convertData(byte[] data){
        int[] output = new int[data.length];
        for(int i=0;i<data.length;i++){
            output[i] = data[i] & 0xff;
        }
        return output;
    }

    private static byte[] convertData(int[] data){
        byte[] output = new byte[data.length];
        for(int i=0;i<data.length;i++){
            output[i] = (byte)data[i];
        }
        return output;
    }
}
