package board.io;

import com.fasterxml.jackson.annotation.JsonIgnore;
import exceptions.MalformedUIDException;

public class InterfaceDescriptor {
    private String uid;
    private int[] intArrayUID;
    private InterfaceType type;

    InterfaceDescriptor(){}

    InterfaceDescriptor(String UID) throws MalformedUIDException {
        if(!this.validateUID(UID)){
            throw new MalformedUIDException();
        }

        this.uid =UID;
        this.intArrayUID = new int[6];
        String[] parts = this.uid.split("\\.");
        int counter = 0;
        for(String part:parts){
            this.intArrayUID[counter] = Integer.parseInt(part);
            counter++;
        }
        this.type = InterfaceType.getType(Integer.parseInt(UID.split("\\.")[0]));
    }

    private boolean validateUID(String UID){
        String[] items = UID.split("\\.");
        if(items.length != 6) return false;
        for(String item:items){
            if(!item.matches("^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$")){
                return false;
            }
        }
        return true;
    }

    public String getUid() {
        return uid;
    }

    @JsonIgnore
    public int[] getIntArrayOfUID(){
        return this.intArrayUID;
    }

    public InterfaceType getType() {
        return type;
    }
}
