/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package main;

import board.Board;
import board.io.Interface;
import board.io.InterfaceDirection;
import board.io.InterfaceSettingsParams;
import com.fazecast.jSerialComm.SerialPort;
import config.Config;
import exceptions.MalformedUIDException;
import exchange.Exchange;
import exchange.dto.BoardConfigMessage;
import exchange.dto.ExchangeMessage;
import exchange.dto.InterfaceConfigMessage;
import exchange.dto.InterfaceStateMessage;
import support.ResourceCache;

import java.time.Instant;
import java.util.*;

/**
 *
 * @author janzubr
 */
public class Main {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args){
        Config.parseArgs(args);
        long startTime = System.currentTimeMillis();
        Exchange E = Exchange.getInstance();
        System.out.println("Boot-time: "+(System.currentTimeMillis()-startTime));

        while(true){
            ExchangeMessage msg = E.fetchMessage();
            Main.processMessage(msg);
        }

    }
    
    private static void processMessage(ExchangeMessage msg){
        if(msg==null) return;
        
        if(msg instanceof BoardConfigMessage){
            Main.processMessage((BoardConfigMessage)msg);
        }

        if(msg instanceof InterfaceStateMessage){
            Main.processMessage((InterfaceStateMessage) msg);
        }
    }
    
    private static void processMessage(BoardConfigMessage msg){
        System.out.println("Processing config message");

        Board.getInstance().setName(msg.getBoardName());
        if(msg.getInterfacesConfig()!=null){
            Map<String, Map<InterfaceSettingsParams,String>> interfacesConfig = msg.getInterfacesConfig().getInterfacesConfig();
            for(Map.Entry<String, Map<InterfaceSettingsParams,String>> interfaceConfig:interfacesConfig.entrySet()){
                try{
                    Interface intf = new Interface(interfaceConfig.getKey());
                    intf.setSettingsParameters(interfaceConfig.getValue());
                    if(Board.getInstance().openInterface(intf)){
                        System.out.println("Opened: "+interfaceConfig.getKey());
                    }
                    else{
                        System.err.println("Open failed "+interfaceConfig.getKey());
                    }
                }
                catch(MalformedUIDException ex){
                    System.err.println("Malformed UID!");
                }
            }
        }

        System.out.println("Processing config message DONE");
    }

    private static void processMessage(InterfaceStateMessage msg){
        for(Map.Entry<String,List<Integer>> entry: msg.getInterfacesStates().entrySet()){
            String uid = entry.getKey();
            int[] data = entry.getValue().stream().mapToInt(Integer::intValue).toArray();
            Board.getInstance().writeInterface(uid,data);
        }
    }
}